@@ -12,68 +12,50 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-import subprocess, sys
+import subprocess
 import os
 
 import gettext
-import locale
-import pygtk
-import gobject
-import gtk
-import gio
-import glib
 import logging
-import pango
+from gi.repository import GObject
+from gi.repository import GLib
+from gi.repository import Gio
+from gi.repository import Gdk
+from gi.repository import Pango
+from gi.repository import Gtk
 
 from usbcreator.frontends.base import Frontend
-from usbcreator.misc import *
+from usbcreator.frontends.gtk.unitysupport import UnitySupport
+from usbcreator import misc
 
 if 'USBCREATOR_LOCAL' in os.environ:
     ui_path = os.path.join(os.getcwd(), 'gui/usbcreator-gtk.ui')
+    eula_path = os.path.join(os.getcwd(), 'gui/ubuntu-nexus7-USAGE-NOTICE-en.txt')
 else:
     ui_path = '/usr/share/usb-creator/usbcreator-gtk.ui'
+    eula_path = '/usr/share/usb-creator/ubuntu-nexus7-USAGE-NOTICE-en.txt'
 
-gtk.gdk.threads_init()
+GObject.threads_init()
+Gdk.threads_init()
 
 def thread_wrap(func):
     '''Decorator for functions that will be called by another thread.'''
     def wrap(*args):
-        gtk.gdk.threads_enter()
+        Gdk.threads_enter()
         try:
             return func(*args)
         finally:
-            gtk.gdk.threads_leave()
+            Gdk.threads_leave()
     return wrap
 
-def wrap_fix(w, allocation):
-    # Until the extended layout branch of GTK+ gets merged (bgo #101968).
-    layout = w.get_layout()
-    old_width, old_height = layout.get_size()
-    if old_width / pango.SCALE == allocation.width:
-        return
-    layout.set_width(allocation.width * pango.SCALE)
-    unused, height = layout.get_size()
-    if old_height != height:
-        w.set_size_request(-1, height / pango.SCALE)
-
-def process_labels(w):
-    if isinstance(w, gtk.Container):
-        children = w.get_children()
-        for c in children:
-            process_labels(c)
-    elif isinstance(w, gtk.Label):
-        if w.get_line_wrap():
-            w.connect_after('size-allocate', wrap_fix)
-        w.set_property('can-focus', False)
-
 class GtkFrontend(Frontend):
     @classmethod
     def startup_failure(cls, message):
-        dialog = gtk.MessageDialog(None, 0, gtk.MESSAGE_ERROR,
-            gtk.BUTTONS_CLOSE, message)
+        dialog = Gtk.MessageDialog(parent=None, flags=0, message_type=Gtk.MessageType.ERROR,
+            buttons=Gtk.ButtonsType.CLOSE, message_format=message)
         dialog.run()
         dialog.destroy()
-        
+
     @classmethod
     def DBusMainLoop(cls):
         from dbus.mainloop.glib import DBusGMainLoop
@@ -83,16 +65,18 @@ class GtkFrontend(Frontend):
                  allow_system_internal=False):
 
         self.allow_system_internal = allow_system_internal
+        self.fastboot_mode = backend.__class__.__name__ == 'FastbootBackend'
 
         self.all_widgets = set()
 
-        self.builder = gtk.Builder()
+        self.builder = Gtk.Builder()
         self.builder.set_translation_domain('usbcreator')
         self.builder.add_from_file(ui_path)
         # A cache of the icon names and display names for the individual
         # devices.
         self.icons = {}
         self.names = {}
+        self.pretty_names = {}
 
         for widget in self.builder.get_objects():
             # Taken from ubiquity:
@@ -102,31 +86,26 @@ class GtkFrontend(Frontend):
             # selectable labels in the focus chain, and I can't seem to turn
             # this off in glade and have it stick. Accordingly, make sure
             # labels are unfocusable here.
-            if isinstance(widget, gtk.Label):
+            if isinstance(widget, Gtk.Label):
                 widget.set_property('can-focus', False)
-            if issubclass(type(widget), gtk.Widget):
+            if issubclass(type(widget), Gtk.Widget):
                 self.all_widgets.add(widget)
-                widget.set_name(gtk.Buildable.get_name(widget))
-                setattr(self, gtk.Widget.get_name(widget), widget)
+                widget.set_name(Gtk.Buildable.get_name(widget))
+                setattr(self, Gtk.Widget.get_name(widget), widget)
 
-        gtk.window_set_default_icon_name('usb-creator-gtk')
-        for w in self.all_widgets:
-            process_labels(w)
+        Gtk.Window.set_default_icon_name('usb-creator-gtk')
 
         # Connect signals to widgets
-        self.builder.connect_signals (self, None)
+        self.builder.connect_signals (self)
         self.cancelbutton.connect('clicked', lambda x: self.warning_dialog.hide())
         self.finished_exit.connect('clicked', lambda x: self.finished_dialog.hide())
         self.failed_exit.connect('clicked', lambda x: self.failed_exit.hide())
         self.progress_cancel_button.connect('clicked', lambda x: self.warning_dialog.show())
-        self.button_help.connect('clicked', lambda x: gtk.show_uri(self.button_help.get_screen(),
-                                                                   'ghelp:usb-creator',
-                                                                   gtk.get_current_event_time()))
 
         def format_value(scale, value):
-            return format_mb_size(value)
+            return misc.format_mb_size(value)
         self.persist_value.set_adjustment(
-            gtk.Adjustment(0, 0, 100, 1, 10, 0))
+            Gtk.Adjustment.new(0, 0, 100, 1, 10, 0))
         self.persist_value.connect('format-value', format_value)
 
         # Connect to backend signals.
@@ -143,12 +122,35 @@ class GtkFrontend(Frontend):
         self.backend.install_progress_pulse_stop_cb = self.progress_pulse_stop
         self.backend.retry_cb = self.retry
         self.backend.target_changed_cb = self.update_target
-        self.backend.format_ended_cb = self.format_ended
         self.backend.format_failed_cb = self.format_failed
 
         # Pulse state.
         self.pulsing = False
 
+        # Hide erase in fastboot mode
+        if self.fastboot_mode:
+            self.format_dest.hide()
+            self.intro_label.set_text(_('To run Ubuntu on a portable device, it needs to be set up first.'))
+            self.label2.set_text(_('Source disc image (.img):'))
+            self.label3.set_text(_('Target device:'))
+            self.window.set_title(_('Ubuntu Core Installer'))
+            self.button_install.set_label(_('Install Ubuntu Core'))
+            self.button_help.set_label(_('Legal'))
+            self.button_help.connect('clicked', self.show_eula)
+            self.eula_dialog.add_buttons(
+                Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,
+                Gtk.STOCK_OK, Gtk.ResponseType.OK
+            )
+            with open(eula_path, 'r') as f:
+                text = ''.join(f.readlines())
+            self.eula_text.get_buffer().set_text(text)
+        else:
+            # we currently do not have help
+            self.button_help.hide()
+            #self.button_help.connect('clicked', lambda x: Gtk.show_uri(self.button_help.get_screen(),
+            #                                                           'ghelp:usb-creator',
+            #                                                           Gtk.get_current_event_time()))
+            
         self.setup_sources_treeview()
         self.setup_targets_treeview()
         self.persist_vbox.set_sensitive(False)
@@ -158,12 +160,13 @@ class GtkFrontend(Frontend):
             self.backend.add_image(img)
             self.source_vbox.hide()
 
-        download_dir = glib.get_user_special_dir(glib.USER_DIRECTORY_DOWNLOAD)
-        if download_dir and os.path.isdir(download_dir):
-            # TODO evand 2009-10-22: File type detection based on file(1).
-            for fname in os.listdir(download_dir):
-                if fname.endswith('.iso') or fname.endswith('.img'):
-                    self.backend.add_image(os.path.join(download_dir, fname))
+        for search_dir_id in [GLib.UserDirectory.DIRECTORY_DOWNLOAD, GLib.UserDirectory.DIRECTORY_DESKTOP]:
+            search_dir = GLib.get_user_special_dir(search_dir_id)
+            if search_dir and os.path.isdir(search_dir):
+                # TODO evand 2009-10-22: File type detection based on file(1).
+                for fname in os.listdir(search_dir):
+                    if fname.endswith('.iso') or fname.endswith('.img'):
+                        self.backend.add_image(os.path.join(search_dir, fname))
 
         # Sets first pre-populated image as current in the backend
         self.selection_changed_source(self.source_treeview.get_selection())
@@ -171,42 +174,41 @@ class GtkFrontend(Frontend):
         if not persistent:
             self.persist_disabled.set_active(True)
             self.persist_vbox.hide()
-        
+
         self.window.show()
-        # Continued from above; hack around GTK's lack of good label wrapping.
-        self.window.set_size_request(self.window.get_allocation().width, -1)
         selection = self.source_treeview.get_selection()
         selection.connect('changed', self.selection_changed_source)
         selection = self.dest_treeview.get_selection()
         selection.connect('changed', self.selection_changed_target)
 
         self.backend.detect_devices()
-        
+        # FIXME: instead of passing parent we really should just send signals
+        self.unity = UnitySupport(parent=self)
         self.update_loop = self.add_timeout(2000, self.backend.update_free)
-        gtk.gdk.threads_enter()
+        Gdk.threads_enter()
         try:
-            gtk.main()
+            Gtk.main()
         except KeyboardInterrupt:
             self.quit()
-        gtk.gdk.threads_leave()
+        Gdk.threads_leave()
 
     def add_timeout(self, interval, func, *args):
         '''Add a new timer for function 'func' with optional arguments. Wraps a
         similar gobject call timeout_add.'''
 
-        timer = gobject.timeout_add(interval, func, *args)
+        timer = GLib.timeout_add(interval, func, *args)
 
         return timer
     def delete_timeout(self, timer):
         '''Remove the specified timer. Wraps gobject source_remove call.'''
 
-        return gobject.source_remove(timer)
+        return GLib.source_remove(timer)
 
     def add_source(self, source):
         logging.debug('add_source: %s' % str(source))
         _append_to_list_and_select(self.source_treeview, [source],
             force_selection=False)
-        
+
         # XXX evand 2009-09-17: Find a label and icon for the device, if we
         # don't already have one.  This will go away when the code for the
         # presentation-name and presentation-icon properties in udisks
@@ -214,7 +216,7 @@ class GtkFrontend(Frontend):
         t = self.backend.sources[source]['type']
         l = self.backend.sources[source]['label']
         d = self.backend.sources[source]['device']
-        if not (t == SOURCE_ISO or t == SOURCE_IMG):
+        if not (t == misc.SOURCE_ISO or t == misc.SOURCE_IMG):
             name, icon = self.get_gnome_drive(d)
             if icon:
                 self.icons[source] = icon
@@ -223,10 +225,13 @@ class GtkFrontend(Frontend):
 
     def add_target(self, target):
         logging.debug('add_target: %s' % str(target))
-        _append_to_list_and_select(self.dest_treeview, [target],
+        # use str() here, since it is originally a DBus.ObjectPath object
+        _append_to_list_and_select(self.dest_treeview, [str(target)],
             force_selection=False)
 
         # XXX evand 2009-09-17: Find a label and icon for the device.
+        v = self.backend.targets[target]['vendor']
+        m = self.backend.targets[target]['model']
         d = self.backend.targets[target]['device']
         l = self.backend.targets[target]['label']
         name, icon = self.get_gnome_drive(d)
@@ -234,6 +239,7 @@ class GtkFrontend(Frontend):
             self.icons[target] = icon
         if not l and name:
             self.names[target] = name
+        self.pretty_names[target] = "%s %s (%s)" % (v, m, misc.text_type(d))
 
     def remove_source(self, source):
         model = self.source_treeview.get_model()
@@ -245,16 +251,14 @@ class GtkFrontend(Frontend):
             iterator = model.iter_next(iterator)
         if to_delete is not None:
             model.remove(to_delete)
-        
+
         if source in self.names:
             self.names.pop(source)
         if source in self.icons:
             self.icons.pop(source)
-        
+
         sel = self.source_treeview.get_selection()
         m, i = sel.get_selected()
-        if not i:
-            sel.select_path(0)
 
     def remove_target(self, target):
         model = self.dest_treeview.get_model()
@@ -266,16 +270,16 @@ class GtkFrontend(Frontend):
             iterator = model.iter_next(iterator)
         if to_delete is not None:
             model.remove(to_delete)
-        
+
         if target in self.names:
             self.names.pop(target)
         if target in self.icons:
             self.icons.pop(target)
-        
+        if target in self.pretty_names:
+            self.pretty_names.pop(target)
+
         sel = self.dest_treeview.get_selection()
         m, i = sel.get_selected()
-        if not i:
-            sel.select_path(0)
 
     def get_source(self):
         '''Returns the UDI of the selected source image.'''
@@ -296,10 +300,10 @@ class GtkFrontend(Frontend):
         else:
             logging.debug('No target selected.')
             return ''
-    
+
     def get_persistence(self):
         if self.persist_enabled.get_active() and \
-            self.persist_enabled.state != gtk.STATE_INSENSITIVE:
+            self.persist_enabled.get_state() != Gtk.StateType.INSENSITIVE:
             val = self.persist_value.get_value()
             return int(val)
         else:
@@ -307,7 +311,7 @@ class GtkFrontend(Frontend):
 
     def get_gnome_drive(self, dev):
         try:
-            monitor = gio.volume_monitor_get()
+            monitor = Gio.VolumeMonitor.get()
             for drive in monitor.get_volumes():
                 if 'unix-device' in drive.enumerate_identifiers():
                     if drive.get_identifier('unix-device') == dev:
@@ -330,7 +334,6 @@ class GtkFrontend(Frontend):
                 return
             udi = model[iterator][0]
             dev = self.backend.sources[udi]
-            t = dev['type']
             if column == 0:
                 if udi in self.names:
                     cell.set_property('text', self.names[udi])
@@ -339,56 +342,59 @@ class GtkFrontend(Frontend):
             elif column == 1:
                 cell.set_property('text', dev['label'])
             elif column == 2:
-                cell.set_property('text', format_size(dev['size']))
+                cell.set_property('text', misc.format_size(dev['size']))
 
-        def pixbuf_data_func(column, cell, model, iterator):
+        def pixbuf_data_func(column, cell, model, iterator, data):
             if not self.backend:
                 return
             udi = model[iterator][0]
             dev = self.backend.sources[udi]
             source_type = dev['type']
-            if source_type == SOURCE_ISO:
-                cell.set_property('stock-id', gtk.STOCK_CDROM)
-            elif source_type == SOURCE_IMG:
-                cell.set_property('stock-id', gtk.STOCK_HARDDISK)
+            if source_type == misc.SOURCE_ISO:
+                cell.set_property('stock-id', Gtk.STOCK_CDROM)
+            elif source_type == misc.SOURCE_IMG:
+                cell.set_property('stock-id', Gtk.STOCK_HARDDISK)
             else:
                 if udi in self.icons:
                     cell.set_property('icon-name', self.icons[udi])
                 else:
                     cell.set_property('stock-id', None)
 
-        list_store = gtk.ListStore(str)
+        list_store = Gtk.ListStore(str)
         self.source_treeview.set_model(list_store)
 
-        cell_name = gtk.CellRendererText()
-        cell_name.set_property('ellipsize', pango.ELLIPSIZE_END)
-        cell_pixbuf = gtk.CellRendererPixbuf()
-        column_name = gtk.TreeViewColumn(_('CD-Drive/Image'))
-        column_name.set_sizing(gtk.TREE_VIEW_COLUMN_AUTOSIZE)
+        cell_name = Gtk.CellRendererText()
+        cell_name.set_property('ellipsize', Pango.EllipsizeMode.END)
+        cell_pixbuf = Gtk.CellRendererPixbuf()
+        column_name = Gtk.TreeViewColumn(_('CD-Drive/Image'))
+        if self.fastboot_mode:
+            column_name = Gtk.TreeViewColumn(_('Image'))
+        column_name.set_sizing(Gtk.TreeViewColumnSizing.AUTOSIZE)
         column_name.set_resizable(True)
         column_name.set_expand(True)
         column_name.set_min_width(75)
-        column_name.pack_start(cell_pixbuf, expand=False)
-        column_name.pack_start(cell_name, expand=True)
+        column_name.pack_start(cell_pixbuf, False)
+        column_name.pack_start(cell_name, True)
         self.source_treeview.append_column(column_name)
         column_name.set_cell_data_func(cell_name, column_data_func, 0)
-        column_name.set_cell_data_func(cell_pixbuf, pixbuf_data_func)
+        column_name.set_cell_data_func(cell_pixbuf, pixbuf_data_func, None)
 
-        cell_version = gtk.CellRendererText()
-        cell_version.set_property('ellipsize', pango.ELLIPSIZE_END)
-        column_name = gtk.TreeViewColumn(_('OS Version'), cell_version)
-        column_name.set_cell_data_func(cell_version, column_data_func, 1)
-        column_name.set_sizing(gtk.TREE_VIEW_COLUMN_AUTOSIZE)
-        column_name.set_resizable(True)
-        column_name.set_expand(True)
-        column_name.set_min_width(75)
-        self.source_treeview.append_column(column_name)
+        if not self.fastboot_mode:
+            cell_version = Gtk.CellRendererText()
+            cell_version.set_property('ellipsize', Pango.EllipsizeMode.END)
+            column_name = Gtk.TreeViewColumn(_('OS Version'), cell_version)
+            column_name.set_cell_data_func(cell_version, column_data_func, 1)
+            column_name.set_sizing(Gtk.TreeViewColumnSizing.AUTOSIZE)
+            column_name.set_resizable(True)
+            column_name.set_expand(True)
+            column_name.set_min_width(75)
+            self.source_treeview.append_column(column_name)
 
-        cell_size = gtk.CellRendererText()
-        cell_size.set_property('ellipsize', pango.ELLIPSIZE_END)
-        column_name = gtk.TreeViewColumn(_('Size'), cell_size)
+        cell_size = Gtk.CellRendererText()
+        cell_size.set_property('ellipsize', Pango.EllipsizeMode.END)
+        column_name = Gtk.TreeViewColumn(_('Size'), cell_size)
         column_name.set_cell_data_func(cell_size, column_data_func, 2)
-        column_name.set_sizing(gtk.TREE_VIEW_COLUMN_AUTOSIZE)
+        column_name.set_sizing(Gtk.TreeViewColumnSizing.AUTOSIZE)
         column_name.set_resizable(True)
         column_name.set_expand(False)
         column_name.set_min_width(75)
@@ -400,13 +406,13 @@ class GtkFrontend(Frontend):
         # files.
 
         def motion_cb(wid, context, x, y, time):
-            context.drag_status(gtk.gdk.ACTION_COPY, time)
+            context.drag_status(Gdk.DragAction.COPY, time)
             return True
-        
+
         def drop_cb(w, context, x, y, time):
             target_list = w.drag_dest_get_target_list()
             target = w.drag_dest_find_target(context, target_list)
-            selection = w.drag_get_data(context, target)
+            w.drag_get_data(context, target)
             context.finish(True, True)
             return True
 
@@ -419,9 +425,9 @@ class GtkFrontend(Frontend):
             elif file.startswith('file:'):
                 file = file[5:]
             self.backend.add_image(file)
-            
-        self.source_treeview.drag_dest_set(gtk.gdk.ACTION_DEFAULT,
-            [('text/uri-list', 0, 600)], gtk.gdk.ACTION_COPY)
+
+        self.source_treeview.drag_dest_set(Gtk.DestDefaults.ALL,
+            [Gtk.TargetEntry.new('text/uri-list', 0, 600)], Gdk.DragAction.COPY)
         self.source_treeview.connect('drag_motion', motion_cb)
         self.source_treeview.connect('drag_drop', drop_cb)
         self.source_treeview.connect('drag-data-received', data_received_cb)
@@ -435,38 +441,41 @@ class GtkFrontend(Frontend):
                 m.row_changed(m.get_path(iterator), iterator)
                 break
             iterator = m.iter_next(iterator)
-        # Update persistence maximum value.
-        self.persist_vbox.set_sensitive(False)
-        self.persist_enabled_vbox.set_sensitive(False)
+
         target = self.backend.targets[udi]
-        persist_mb = target['persist'] / 1024 / 1024
-        if persist_mb > MIN_PERSISTENCE:
-                self.persist_vbox.set_sensitive(True)
-                self.persist_enabled_vbox.set_sensitive(True)
-                self.persist_value.set_range(MIN_PERSISTENCE, persist_mb)
-                self.persist_value.set_value(MIN_PERSISTENCE)
+
+        # Update persistence maximum value.
+        if not self.fastboot_mode:
+            self.persist_vbox.set_sensitive(False)
+            self.persist_enabled_vbox.set_sensitive(False)
+            persist_mb = target['persist'] / 1024 / 1024
+            if persist_mb > misc.MIN_PERSISTENCE:
+                    self.persist_vbox.set_sensitive(True)
+                    self.persist_enabled_vbox.set_sensitive(True)
+                    self.persist_value.set_range(misc.MIN_PERSISTENCE, persist_mb)
+                    self.persist_value.set_value(misc.MIN_PERSISTENCE)
+
         # Update install button state.
         status = target['status']
         source = self.backend.get_current_source()
         if not source:
             self.button_install.set_sensitive(False)
             return
-        stype = self.backend.sources[source]['type']
-        if status == CAN_USE:
+        if status == misc.CAN_USE:
             self.button_install.set_sensitive(True)
         else:
             self.button_install.set_sensitive(False)
         # Update the destination status message.
         self.open_dest.hide()
-        if status == CANNOT_USE:
+        if status == misc.CANNOT_USE:
             msg = _('The device is not large enough to hold this image.')
-        elif status == NEED_SPACE:
+        elif status == misc.NEED_SPACE:
             msg = _('There is not enough free space for this image.')
             self.open_dest.show()
         else:
             msg = ''
         self.dest_status.set_text(msg)
-        
+
     def selection_changed_source(self, selection):
         model, iterator = selection.get_selected()
         if not iterator:
@@ -479,12 +488,31 @@ class GtkFrontend(Frontend):
         '''The selected partition has changed and the bounds on the persistence
         slider need to be changed, or the slider needs to be disabled, to
         reflect the amount of free space on the partition.'''
-        
+
         model, iterator = selection.get_selected()
         if not iterator:
             return
         udi = model[iterator][0]
-        self.update_target(udi)
+        if udi:
+            self.update_target(udi)
+
+        if self.fastboot_mode:
+            return
+
+        dev = self.backend.targets[udi]
+        p = dev['parent']
+        if p and p in self.backend.targets:
+            dev = self.backend.targets[p]
+        if dev['formatting']:
+            self.format_dest.set_sensitive(False)
+            self.format_dest_label.set_text(_('Erasing...'))
+            self.format_dest_spinner.show()
+            self.format_dest_spinner.start()
+        else:
+            self.format_dest.set_sensitive(True)
+            self.format_dest_label.set_text(_('Erase Disk'))
+            self.format_dest_spinner.hide()
+            self.format_dest_spinner.stop()
 
     def setup_targets_treeview(self):
         def column_data_func(layout, cell, model, iterator, column):
@@ -492,114 +520,126 @@ class GtkFrontend(Frontend):
                 return
             udi = model[iterator][0]
             dev = self.backend.targets[udi]
-            
+
             if column == 0:
-                cell.set_property('text', dev['device'])
+                if udi in self.pretty_names:
+                    cell.set_property('text', self.pretty_names[udi])
+                else:
+                    cell.set_property('text', dev['device'])
             elif column == 1:
                 if udi in self.names:
                     cell.set_property('text', self.names[udi])
                 else:
                     cell.set_property('text', dev['label'])
             elif column == 2:
-                cell.set_property('text', format_size(dev['capacity']))
+                cell.set_property('text', misc.format_size(dev['capacity']))
             elif column == 3:
                 free = dev['free']
                 if free >= 0:
-                    cell.set_property('text', format_size(free))
+                    cell.set_property('text', misc.format_size(free))
                 else:
                     cell.set_property('text', '')
 
-        def pixbuf_data_func(column, cell, model, iterator):
+        def pixbuf_data_func(column, cell, model, iterator, data):
             if not self.backend:
                 return
             udi = model[iterator][0]
             dev = self.backend.targets[udi]
             status = dev['status']
 
-            if status == NEED_SPACE:
-                cell.set_property('stock-id', gtk.STOCK_DIALOG_WARNING)
-            elif status == CANNOT_USE:
+            if status == misc.NEED_SPACE:
+                cell.set_property('stock-id', Gtk.STOCK_DIALOG_WARNING)
+            elif status == misc.CANNOT_USE:
                 # TODO evand 2009-05-05: Implement disabled rows as a
                 # replacement?
-                cell.set_property('stock-id', gtk.STOCK_DIALOG_ERROR)
+                cell.set_property('stock-id', Gtk.STOCK_DIALOG_ERROR)
             else:
                 if udi in self.icons:
                     cell.set_property('icon-name', self.icons[udi])
                 else:
                     cell.set_property('stock-id', None)
 
-        list_store = gtk.ListStore(str)
-        list_store.set_sort_column_id(0, gtk.SORT_ASCENDING)
+        list_store = Gtk.ListStore(GObject.TYPE_STRING)
+        list_store.set_sort_column_id(0, Gtk.SortType.ASCENDING)
         self.dest_treeview.set_model(list_store)
 
-        column_name = gtk.TreeViewColumn()
+        column_name = Gtk.TreeViewColumn()
         column_name.set_title(_('Device'))
-        cell_name = gtk.CellRendererText()
-        cell_name.set_property('ellipsize', pango.ELLIPSIZE_END)
-        cell_pixbuf = gtk.CellRendererPixbuf()
-        column_name.set_sizing(gtk.TREE_VIEW_COLUMN_AUTOSIZE)
+        cell_name = Gtk.CellRendererText()
+        cell_name.set_property('ellipsize', Pango.EllipsizeMode.END)
+        cell_pixbuf = Gtk.CellRendererPixbuf()
+        column_name.set_sizing(Gtk.TreeViewColumnSizing.AUTOSIZE)
         column_name.set_resizable(True)
         column_name.set_expand(True)
         column_name.set_min_width(75)
-        column_name.pack_start(cell_pixbuf, expand=False)
-        column_name.pack_start(cell_name, expand=True)
+        column_name.pack_start(cell_pixbuf, False)
+        column_name.pack_start(cell_name, True)
         self.dest_treeview.append_column(column_name)
         column_name.set_cell_data_func(cell_name, column_data_func, 0)
-        column_name.set_cell_data_func(cell_pixbuf, pixbuf_data_func)
-        
-        cell_name = gtk.CellRendererText()
-        cell_name.set_property('ellipsize', pango.ELLIPSIZE_END)
-        column_name = gtk.TreeViewColumn(_('Label'), cell_name)
-        column_name.set_cell_data_func(cell_name, column_data_func, 1)
-        column_name.set_sizing(gtk.TREE_VIEW_COLUMN_AUTOSIZE)
-        column_name.set_resizable(True)
-        column_name.set_expand(True)
-        column_name.set_min_width(75)
-        self.dest_treeview.append_column(column_name)
+        column_name.set_cell_data_func(cell_pixbuf, pixbuf_data_func, None)
 
-        cell_capacity = gtk.CellRendererText()
-        cell_capacity.set_property('ellipsize', pango.ELLIPSIZE_END)
-        column_name = gtk.TreeViewColumn(_('Capacity'), cell_capacity)
-        column_name.set_cell_data_func(cell_capacity, column_data_func, 2)
-        column_name.set_sizing(gtk.TREE_VIEW_COLUMN_AUTOSIZE)
-        column_name.set_resizable(True)
-        column_name.set_expand(False)
-        column_name.set_min_width(75)
-        self.dest_treeview.append_column(column_name)
+        if not self.fastboot_mode:
+            cell_name = Gtk.CellRendererText()
+            cell_name.set_property('ellipsize', Pango.EllipsizeMode.END)
+            column_name = Gtk.TreeViewColumn(_('Label'), cell_name)
+            column_name.set_cell_data_func(cell_name, column_data_func, 1)
+            column_name.set_sizing(Gtk.TreeViewColumnSizing.AUTOSIZE)
+            column_name.set_resizable(True)
+            column_name.set_expand(True)
+            column_name.set_min_width(75)
+            self.dest_treeview.append_column(column_name)
 
-        cell_free = gtk.CellRendererText()
-        cell_free.set_property('ellipsize', pango.ELLIPSIZE_END)
-        column_name = gtk.TreeViewColumn(_('Free Space'), cell_free)
-        column_name.set_cell_data_func(cell_free, column_data_func, 3)
-        column_name.set_sizing(gtk.TREE_VIEW_COLUMN_AUTOSIZE)
-        column_name.set_resizable(True)
-        column_name.set_expand(False)
-        column_name.set_min_width(75)
-        self.dest_treeview.append_column(column_name)
+            cell_capacity = Gtk.CellRendererText()
+            cell_capacity.set_property('ellipsize', Pango.EllipsizeMode.END)
+            column_name = Gtk.TreeViewColumn(_('Capacity'), cell_capacity)
+            column_name.set_cell_data_func(cell_capacity, column_data_func, 2)
+            column_name.set_sizing(Gtk.TreeViewColumnSizing.AUTOSIZE)
+            column_name.set_resizable(True)
+            column_name.set_expand(False)
+            column_name.set_min_width(75)
+            self.dest_treeview.append_column(column_name)
+
+            cell_free = Gtk.CellRendererText()
+            cell_free.set_property('ellipsize', Pango.EllipsizeMode.END)
+            column_name = Gtk.TreeViewColumn(_('Free Space'), cell_free)
+            column_name.set_cell_data_func(cell_free, column_data_func, 3)
+            column_name.set_sizing(Gtk.TreeViewColumnSizing.AUTOSIZE)
+            column_name.set_resizable(True)
+            column_name.set_expand(False)
+            column_name.set_min_width(75)
+            self.dest_treeview.append_column(column_name)
 
     def add_file_source_dialog(self, *args):
         filename = ''
-        chooser = gtk.FileChooserDialog(title=None,action=gtk.FILE_CHOOSER_ACTION_OPEN,
-            buttons=(gtk.STOCK_CANCEL,gtk.RESPONSE_CANCEL,gtk.STOCK_OPEN,gtk.RESPONSE_OK))
-        for p, n in (('*.iso', _('CD Images')), ('*.img', _('Disk Images'))):
-            filter = gtk.FileFilter()
+        chooser = Gtk.FileChooserDialog(title=None,action=Gtk.FileChooserAction.OPEN,
+            buttons=(Gtk.STOCK_CANCEL,Gtk.ResponseType.CANCEL,Gtk.STOCK_OPEN,Gtk.ResponseType.OK))
+        def _add_filter(p, n):
+            filter = Gtk.FileFilter()
             filter.add_pattern(p)
             filter.set_name(n)
             chooser.add_filter(filter)
+
+        if not self.fastboot_mode:
+            _add_filter('*.iso', _('CD Images'))
+        _add_filter('*.img', _('Disk Images'))            
+
         folder = os.path.expanduser('~')
         chooser.set_current_folder(folder)
         response = chooser.run()
-        if response == gtk.RESPONSE_OK:
+        if response == Gtk.ResponseType.OK:
             filename = chooser.get_filename()
             chooser.destroy()
             self.backend.add_image(filename)
-        elif response == gtk.RESPONSE_CANCEL:
+        elif response == Gtk.ResponseType.CANCEL:
             chooser.destroy()
 
     def install(self, widget):
         source = self.get_source()
         target = self.get_target()
         persist = self.get_persistence()
+        if self.fastboot_mode and not misc.check_eula():    
+            if self.show_eula() == Gtk.ResponseType.CANCEL:
+                return
         # TODO evand 2009-07-31: Make these the default values in the
         # GtkBuilder file.
         starting_up = _('Starting up...')
@@ -608,6 +648,7 @@ class GtkFrontend(Frontend):
         if source and target:
             self.install_window.show()
             self.window.hide()
+            self.unity.show_progress()
             self.delete_timeout(self.update_loop)
             try:
                 self.backend.install(source, target, persist,
@@ -624,6 +665,7 @@ class GtkFrontend(Frontend):
         if complete > 100:
             complete = 100
         self.progress_bar.set_fraction(complete / 100.0)
+        self.unity.set_progress(complete / 100.0)
         if remaining and speed:
             # TODO evand 2009-07-24: Could use a time formatting function
             # like our human size function.
@@ -654,66 +696,86 @@ class GtkFrontend(Frontend):
 
     @thread_wrap
     def retry(self, message):
-        retry_dialog = gtk.MessageDialog(self.window, gtk.DIALOG_MODAL |
-                                         gtk.DIALOG_DESTROY_WITH_PARENT,
-                                         gtk.MESSAGE_ERROR, gtk.BUTTONS_YES_NO,
-                                         message)
+        retry_dialog = Gtk.MessageDialog(parent=self.window, flags=Gtk.DialogFlags.MODAL |
+                                         Gtk.DialogFlags.DESTROY_WITH_PARENT,
+                                         message_type=Gtk.MessageType.ERROR, buttons=Gtk.ButtonsType.YES_NO,
+                                         message_format=message)
         response = retry_dialog.run()
         retry_dialog.destroy()
-        return response == gtk.RESPONSE_YES
+        return response == Gtk.ResponseType.YES
+
+    def show_eula(self, unused=None):
+        response = self.eula_dialog.run()
+        if response == Gtk.ResponseType.OK:
+            misc.check_eula(True)
+        self.eula_dialog.hide()
+        return response
 
-    
     def quit(self, *args):
         self.backend.cancel_install()
-        if gtk.main_level() > 0:
-            gtk.main_quit()
+        if Gtk.main_level() > 0:
+            Gtk.main_quit()
 
     @thread_wrap
     def failure(self, message=None):
         self._fail(message)
-    
+
     def _fail(self, message=None):
         logging.exception('Installation failed.')
         # FIXME: evand 2009-07-28: Do we need this?
         self.warning_dialog.hide()
         self.install_window.hide()
+        self.unity.show_progress(False)
         if not message:
             message = _('Installation failed.')
         self.failed_dialog_label.set_text(message)
         self.failed_dialog.run()
-        gtk.main_quit()
-    
+        Gtk.main_quit()
+
     @thread_wrap
     def success(self):
         self.backend.unmount()
         self.warning_dialog.hide()
         self.install_window.hide()
+        self.unity.show_progress(False)
+        try:
+            import dbus
+            bus = dbus.SystemBus()
+            obj = bus.get_object('com.ubuntu.USBCreator',
+                                 '/com/ubuntu/USBCreator')
+            if obj.KVMOk(dbus_interface='com.ubuntu.USBCreator',
+                         timeout=misc.MAX_DBUS_TIMEOUT):
+                self.kvm_test.show()
+                def go(*args):
+                    obj.KVMTest(self.get_target(), os.environ,
+                                dbus_interface='com.ubuntu.USBCreator')
+                self.kvm_test.connect('clicked', go)
+        except dbus.DBusException:
+            logging.exception('Error checking for kvm:')
+
+        if self.fastboot_mode:
+            self.finished_dialog_label.set_text(
+                _('Installation is complete. Your device is rebooting into Ubuntu Core.'))
+            self.kvm_test.hide()
         self.finished_dialog.run()
-        gtk.main_quit()
+        self.backend.shutdown()
+        Gtk.main_quit()
 
     def notify(self, message):
-        dialog = gtk.MessageDialog(self.window, 0, gtk.DIALOG_MODAL |
-                                   gtk.DIALOG_DESTROY_WITH_PARENT,
-                                   gtk.MESSAGE_WARNING, gtk.BUTTONS_CLOSE,
-                                   message)
+        dialog = Gtk.MessageDialog(parent=self.window, flags=Gtk.DialogFlags.MODAL |
+                                   Gtk.DialogFlags.DESTROY_WITH_PARENT,
+                                   message_type=Gtk.MessageType.WARNING, buttons=Gtk.ButtonsType.CLOSE,
+                                   message_format=message)
         dialog.run()
         dialog.destroy()
 
-    def format_ended(self):
-        self.backend.formatting = None
-        self.window.window.set_cursor(None)
-        self.format_dest.set_sensitive(True)
-
     @thread_wrap
     def format_failed(self, message):
-        self.backend.formatting = None
-        self.window.window.set_cursor(None)
-        self.format_dest.set_sensitive(True)
         # TODO sort through error types (message.get_dbus_name()) in backend,
         # individual functions in frontend for each error type.
-        d = gtk.MessageDialog(self.window, gtk.DIALOG_MODAL,
-                              type=gtk.MESSAGE_ERROR,
-                              buttons=gtk.BUTTONS_CLOSE)
+        d = Gtk.MessageDialog(parent=self.window, flags=Gtk.DialogFlags.MODAL,
+                              message_type=Gtk.MessageType.ERROR,
+                              buttons=Gtk.ButtonsType.CLOSE)
         d.set_markup(str(message))
         d.run()
         d.destroy()
@@ -723,17 +785,17 @@ class GtkFrontend(Frontend):
         if not iterator:
             return
         udi = model[iterator][0]
-        d = gtk.MessageDialog(self.window, gtk.DIALOG_MODAL,
-                type=gtk.MESSAGE_QUESTION, buttons=gtk.BUTTONS_YES_NO)
+        d = Gtk.MessageDialog(parent=self.window, flags=Gtk.DialogFlags.MODAL,
+                message_type=Gtk.MessageType.QUESTION, buttons=Gtk.ButtonsType.YES_NO)
         # TODO information about the device we're about to format.
         d.set_markup(_('Are you sure you want to erase the entire disk?'))
         response = d.run()
         d.destroy()
-        if response == gtk.RESPONSE_YES:
-            self.window.window.set_cursor(gtk.gdk.Cursor(gtk.gdk.WATCH))
-            # While we can support formatting multiple devices at once, lets
-            # not for simplicity's sake.
+        if response == Gtk.ResponseType.YES:
             self.format_dest.set_sensitive(False)
+            self.format_dest_label.set_text(_('Erasing...'))
+            self.format_dest_spinner.show()
+            self.format_dest_spinner.start()
             self.backend.format(udi)
 
     def open_dest_folder(self, *args):
@@ -750,6 +812,6 @@ def _append_to_list_and_select(treeview, new_row, force_selection):
     model = treeview.get_model()
     new_iter = model.append(new_row)
     if force_selection or (treeview.get_selection().get_selected()[1] is None):
-        treeview.set_cursor(model.get_path(new_iter))
+        treeview.set_cursor(model.get_path(new_iter), None, False)
 
 # vim: set ai et sts=4 tabstop=4 sw=4:
